/*
Copyright © 2025 filifa

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package depgraph

import (
	"hash/crc32"

	"gonum.org/v1/gonum/graph"
	"gonum.org/v1/gonum/graph/simple"
)

// DependencyGraph represents dependencies between files in a Go package.
type DependencyGraph struct {
	*simple.DirectedGraph
}

// NewDependencyGraph returns an empty DependencyGraph.
func NewDependencyGraph() *DependencyGraph {
	return &DependencyGraph{DirectedGraph: simple.NewDirectedGraph()}
}

// NewNode returns a new Sourcefile node.
func (g *DependencyGraph) NewNode() graph.Node {
	return &Sourcefile{Node: g.DirectedGraph.NewNode()}
}

// Sourcefile represents a given source file in a Go package.
type Sourcefile struct {
	graph.Node
	name string
}

// String returns the name attribute of Sourcefile.
func (s *Sourcefile) String() string {
	return s.name
}

// NewSourcefile returns a Sourcefile node with ID determined by the CRC32 hash
// of the name of the source file.
func NewSourcefile(name string) *Sourcefile {
	hash := crc32.ChecksumIEEE([]byte(name))
	return &Sourcefile{Node: simple.Node(hash), name: name}
}

// DOTID returns the name attribute of the source file.
func (s *Sourcefile) DOTID() string {
	return s.name
}

// SetDOTID sets the name attribute of a Sourcefile to the given id.
func (s *Sourcefile) SetDOTID(id string) {
	s.name = id
}
